/*
 *  cs.c
 *
 *  Sending and receiving packets. CS means client and server.
 *
 *  Copyright (C) 2008  Huang Guan
 *
 *  2008-01-31 Created.
 *
 *  Description: This file mainly includes the functions about 
 *               Sending Command Packet
 *               Receving Command Packet
 *               Sending Keep-Alive Message && Handling Timeout Packet
 *
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#ifdef __WIN32__
#include <winsock.h>
#include <wininet.h>
#else
#include <sys/socket.h>
#include <arpa/inet.h>
#endif

#include "myqq.h"
#include "packet.h"
#include "protocol.h"
#include "crypt.h"
#include "debug.h"

//Send a command to the server
//success:>0  failure:-1
int cs_send_cmd_with_seqno( int qid, ushort cmd, char *data, size_t len, int ack, int seqno)
{
	int rlen, ret;
	myqq_prot_packet_t* pout = packet_alloc( qid, MAX_PACKET_SIZE, PACKET_OUT );
	pout->seqno = htons(seqno);
	pout->command = htons( cmd );
	rlen = len;
	if( MYQQ(qid).status == ST_ONLINE )
	{
		//Use session key to encrypt our data.
		qq_crypt( ENCRYPT, (unsigned char*)data, len, 
			(unsigned char*)MYQQ(qid).session_key, (unsigned char*)pout->data_out, &rlen );
	}else{
		memcpy( pout->data_out, data, rlen );
	}
	pout->data_out[rlen] = PROT_TAIL;
	ret = packet_send( qid, pout, rlen+1+11 );
	if( !ack )
		packet_free( qid, pout );
	return ret;
}

int cs_send_cmd( int qid, ushort cmd, char *data, size_t len, int ack)
{
	return cs_send_cmd_with_seqno( qid, cmd, data, len, ack, MYQQ(qid).seqno++ );
}


//յqqϢ
static void cs_recv_packet( int qid, myqq_prot_packet_t *p_in, int len_in )
{
	int rlen, data_len;
	myqq_prot_packet_t *p_out;
	if( ( p_out = packet_match( qid, p_in ) ) )
	{
		//Ƿһظ
		//DBG("Ack packet recved.");
	}else{
		//Ƿһ
		//DBG("Server Require sth.");
	}
	//Now decript the data
	char* data = (char*)malloc( MAX_PACKET_SIZE );
	rlen = len_in - 8;
	data_len = MAX_PACKET_SIZE;
	if( MYQQ(qid).status != ST_OFFLINE )
	{
		//Use session key to decrypt our data.
		qq_crypt( DECRYPT, (unsigned char*)p_in->data_in, rlen, 
			(unsigned char*)MYQQ(qid).session_key, (unsigned char*)data, &data_len );
	}
	switch( ntohs( p_in->command ) ){
	case QQ_CMD_LOGIN:	//¼
		DBG("QQ_CMD_LOGIN");
		login_recv_info( qid, p_in->data_in, rlen );
		break;
	case QQ_CMD_REQUEST_LOGIN_TOKEN:
		DBG("QQ_CMD_REQUEST_LOGIN_TOKEN");
		break;
	case QQ_CMD_KEEP_ALIVE:
		DBG("QQ_CMD_KEEP_ALIVE");
		cs_recv_keep_alive( qid, data, data_len );
		break;
	case QQ_CMD_UPDATE_INFO:
		DBG("QQ_CMD_UPDATE_INFO");
		break;
	case QQ_CMD_SEARCH_USER:
		DBG("QQ_CMD_SEARCH_USER");
		break;
	case QQ_CMD_GET_USER_INFO:	//ȡϢԼ
		DBG("QQ_CMD_GET_USER_INFO");
		buddy_recv_info( qid, data, data_len ); //buddy.c
		break;
	case QQ_CMD_ADD_FRIEND_WO_AUTH:
		DBG("QQ_CMD_ADD_FRIEND_WO_AUTH");
		break;
	case QQ_CMD_DEL_FRIEND:
		DBG("QQ_CMD_DEL_FRIEND");
		break;
	case QQ_CMD_BUDDY_AUTH:
		DBG("QQ_CMD_BUDDY_AUTH");
		break;
	case QQ_CMD_CHANGE_ONLINE_STATUS:
		DBG("QQ_CMD_CHANGE_ONLINE_STATUS");
		login_change_status_comp( qid, data, data_len );
		break;
	case QQ_CMD_ACK_SYS_MSG:
		DBG("QQ_CMD_ACK_SYS_MSG");
		break;
	case QQ_CMD_SEND_IM:
		DBG("QQ_CMD_SEND_IM");
		im_send_comp( qid, data, data_len );
		break;
	case QQ_CMD_RECV_IM:
		DBG("QQ_CMD_RECV_IM");
		im_recv( qid, data, data_len, p_in );	//im.c
		break;
	case QQ_CMD_REMOVE_SELF:
		DBG("QQ_CMD_REMOVE_SELF");
		break;
	case QQ_CMD_REQUEST_KEY:
		DBG("QQ_CMD_REQUEST_KEY");
		break;
	case QQ_CMD_CELL_PHONE_1:
		DBG("QQ_CMD_CELL_PHONE_1");
		break;
	case QQ_CMD_GET_FRIENDS_LIST:
		DBG("QQ_CMD_GET_FRIENDS_LIST");
		buddy_recv_list( qid, data, data_len );
		break;
	case QQ_CMD_GET_FRIENDS_ONLINE:
		DBG("QQ_CMD_GET_FRIENDS_ONLINE");
		buddy_recv_online_list( qid, data, data_len );
		break;
	case QQ_CMD_CELL_PHONE_2:
		DBG("QQ_CMD_CELL_PHONE_2");
		break;
	case QQ_CMD_GROUP_CMD:
		DBG("QQ_CMD_GROUP_CMD");
		group_recv_info( qid, data, data_len );
		break;
	case QQ_CMD_GET_ALL_LIST_WITH_GROUP:
		DBG("QQ_CMD_GET_ALL_LIST_WITH_GROUP");
		group_recv_list( qid, data, data_len );
		break;
	case QQ_CMD_GET_LEVEL:
		DBG("QQ_CMD_GET_LEVEL");
		break;
	case QQ_CMD_RECV_MSG_SYS:
		DBG("QQ_CMD_RECV_MSG_SYS");
		break;
	case QQ_CMD_RECV_MSG_FRIEND_CHANGE_STATUS:
		DBG("QQ_CMD_RECV_MSG_FRIEND_CHANGE_STATUS");
		buddy_recv_status( qid, data, data_len );
		break;
	default:
		DBG("UNKNOWN PACKET!!: cmd=%d", p_in->command );
		break;
	}
	free( data );
}


//a loop for receiving packets
static void recv_packet_loop( void* data )
{
	int len_in;
	int qid = (int)data;
	DBG("%d start recving packet.", qid );
	while( 1 )
	{
		myqq_prot_packet_t *packet_in = packet_alloc( qid, MAX_PACKET_SIZE, PACKET_IN );
		len_in = packet_recv( qid, packet_in, MAX_PACKET_SIZE );
		if( len_in > 0 )
		{
			if( packet_in->headflag != PROT_HEAD )
			{
				DBG("Bad packet.");
				packet_free( qid, packet_in );
				continue;
			}
			cs_recv_packet( qid, packet_in, len_in );
		}
		packet_free( qid, packet_in );
	}
	//nerver reach here.
}

int cs_start_recv_packet( int qid )
{
	int ret;
	ret = pthread_create( &MYQQ(qid).thread_recv, NULL, (void*)recv_packet_loop, (void*)qid );
	if( ret != 0 )
	{
		DBG("Create thread ERROR.");
		return -EFAULT;
	}
	return 0;
}

void cs_end_recv_packet( int qid )
{
	pthread_cancel( MYQQ(qid).thread_recv );
	//MYQQ(qid).thread_recv = (pthread_t)(0);  //cause error in mingwgcc
}



//For keeping online, we have to send KEEP_ALIVE message every 30secs
#define INTERNAL 1
static void cs_keep_alive( void* data )
{
	int qid = (int)data;
	int uid = htonl( MYQQ(qid).uid );
	int counter = 1;
	while( MYQQ(qid).status == ST_ONLINE )
	{
#ifdef __WIN32__	//WindowsʱLinuxĿ1000
		sleep( INTERNAL*1000 ); //sleep for 1 secs 
#else
		sleep( INTERNAL ); //sleep for 1 secs 
#endif
		counter ++;
		if( counter%30 == 0 ) //send every 30secs
		{
			cs_send_cmd( qid, QQ_CMD_KEEP_ALIVE, (char*)&uid, 4, 1 );
		}
		//find overtime packet
		myqq_prot_packet_t *packet;
		int packet_len, times;
		while( (packet = packet_timeout( qid, 5, &packet_len, &times )) )
		{
			if( times > 3 )
			{
				if( ntohs( packet->command ) < 0x100 )
					MSG("Packet CMD:%d has not been sent.\n", 
						ntohs( packet->command ) );
				else
					DBG("Packet CMD:%d has not been sent.\n", 
						ntohs( packet->command ) );
				packet_free( qid, packet );
			}else{
				packet_send( qid, packet, packet_len );
			}
		}
	}
}

void cs_recv_keep_alive( int qid, char* data, int len )
{
	//data is seperated by 0x1f, we are only interested in the third one.
	/*  NOT IMPLEMENTED  */
	//hex_dump( data, len );
}



int cs_start_keep_alive( int qid )
{
	int ret;
	ret = pthread_create( &MYQQ(qid).thread_alive, NULL, (void*)cs_keep_alive, (void*)qid );
	if( ret != 0 )
	{
		DBG("Create thread ERROR.");
		return -EFAULT;
	}
	return 0;
}

void cs_end_keep_alive( int qid )
{
	pthread_cancel( MYQQ(qid).thread_alive );
	//MYQQ(qid).thread_recv = (pthread_t)(0);  //cause error in mingwgcc
}

//wait for all packets sent.
EXPORT int myqq_wait( int qid, int timeout )
{
	int len, times, i;
	for( i=0; timeout==0 || i < timeout; i++ )
	{
		if( packet_timeout( qid, 0, &len, &times ) == NULL )
			return 0;
#ifdef __WIN32__
		sleep( 1000 );
#else
		sleep( 1 );
#endif
	}
	return -1;	//timeout
}

