/*
 *  im.c
 *
 *  Some functions for sending and receiving messages.
 *
 *  Copyright (C) 2008  Huang Guan
 *
 *  2008-01-31 Created.
 *
 *  Description: This file mainly includes the functions about 
 *               Sending IM ( Group IM is NOT included )
 *               Receving IM ( Group IM is NOT included )
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef __WIN32__
#include <winsock.h>
#else
#include <sys/socket.h>
#include <arpa/inet.h>
#endif

#include "myqq.h"
#include "packet.h"
#include "protocol.h"
#include "cs.h"
#include "debug.h"



//发送16字节给服务器验证已经收到。
static void im_ack( int qid, char *data, int seqno )
{
	cs_send_cmd_with_seqno( qid, QQ_CMD_RECV_IM, data, 16, 0, seqno );
}

//sometime we will receiving system message.
//e.g. Login twice.
static void im_recv_sys_msg( int qid, char* data, int len )
{
	int msg_len = data[1];
	char* msg = (char*)malloc( msg_len + 1 );
	if( msg ){
		memcpy( msg, &data[2], msg_len );
		msg[msg_len] = 0;
		MSG("\nSystem Message: %s\n", msg );
	}
}

//处理接收到的消息
void im_recv( int qid, char* data, int len, myqq_prot_packet_t* packet )
{
	myqq_prot_recv_im_t* im = (myqq_prot_recv_im_t*)data;
	char* next = data + sizeof(myqq_prot_recv_im_t);
	int next_len = len - sizeof(myqq_prot_recv_im_t);
	//回应服务器
	im_ack( qid, data, ntohs(packet->seqno) );
	switch( ntohs(im->type) ){
	case QQ_RECV_IM_TO_BUDDY:	//好友消息
		DBG("QQ_RECV_IM_TO_BUDDY");
		buddy_recv_im( qid, next, next_len );
		break;
	case QQ_RECV_IM_SYS_NOTIFICATION:	//系统消息
		DBG("QQ_RECV_IM_SYS_NOTIFICATION");
		im_recv_sys_msg( qid, next, next_len );
		break;
	case QQ_RECV_IM_QUN_IM:	//群消息
	case QQ_RECV_IM_TEMP_QUN_IM:	//临时群消息
	case QQ_RECV_IM_UNKNOWN_QUN_IM:	//未知群的消息
		DBG("QQ_RECV_IM_QUN_IM");
		group_recv_im( qid, data, len, ntohs(im->type) );
		break;
	case QQ_RECV_IM_CREATE_QUN:		//新创建群通知
		DBG("QQ_RECV_IM_CREATE_QUN");
		break;
	case QQ_RECV_IM_REJCT_APPLY_ADD_TO_QUN:	//加入群请求拒绝通知
		DBG("QQ_RECV_IM_REJCT_APPLY_ADD_TO_QUN");
		break;
	case QQ_RECV_IM_APPROVE_APPLY_ADD_TO_QUN:		//加入群请求批准通知
		DBG("QQ_RECV_IM_APPROVE_APPLY_ADD_TO_QUN");
		break;
	case QQ_RECV_IM_APPLY_ADD_TO_QUN:	//请求批准加入群通知
		DBG("QQ_RECV_IM_APPLY_ADD_TO_QUN");
		break;
	case QQ_RECV_IM_DEL_FROM_QUN:	//被从群删除通知
		DBG("QQ_RECV_IM_DEL_FROM_QUN");
		break;
	case QQ_RECV_IM_ADD_TO_QUN:		//加入群通知
		DBG("QQ_RECV_IM_ADD_TO_QUN");
		break;
	case QQ_RECV_IM_TO_UNKNOWN:		//到未知群的消息
		DBG("QQ_RECV_IM_TO_UNKNOWN");
		break;
	default:
		DBG("Unknown Message type: %d", ntohs(im->type) );
	}
}


void im_send( int qid, uint uid, char* content, int len )
{
	char* data = malloc( len+16 );
	uint sender_uid = htonl( MYQQ(qid).uid );
	uint recver_uid = htonl( uid );
	memcpy( data, &sender_uid, 4 );
	memcpy( data+4, &recver_uid, 4 );
	memcpy( data+8, content, len );
	cs_send_cmd( qid, QQ_CMD_SEND_IM, data, 8+len, 1 );
	DBG("send message to %d.", uid );
	free( data );
}

void im_send_comp( int qid, char* data, int len )
{
	if( len!=1 || data[0]!=0 )
	{
		MSG("Send message error.");
	}
}
