/*
 *  MYQQ(qid).c
 *
 *  main() A small console for MyQQ.
 *
 *  Copyright (C) 2008  Huang Guan
 *
 *  2008-01-31 Created.
 *  2008-2-5   Add more commands.
 *
 *  Description: This file mainly includes the functions about 
 *               Parsing Input Commands.
 *               
 */


#include <stdio.h>
#include <string.h>
#include <time.h>
#include <stdlib.h>
#include <unistd.h>
#include "myqqdef.h"

#define MSG need_reset = 1; printf

#define GROUP_BUF_SIZE	80*100
#define BUDDY_BUF_SIZE	80*500


static char* group_buf, *buddy_buf;
static uint to_uid = 0;		//send message to that guy.
static uint group_int_uid;		//internal group id if entered.
static char input[1024];
static int enter = 0;
static int qid;
static int need_reset;
enum{
	CMD_EXIT = 0, CMD_EXIT2,
	CMD_SAY, CMD_SAY2,
	CMD_TO, CMD_TO2,
	CMD_HELP,
	CMD_STATUS,
	CMD_ENTER, CMD_ENTER2,
	CMD_LEAVE, CMD_LEAVE2,
	CMD_WHO, CMD_WHO2,
	CMD_VIEW, CMD_VIEW2,
	CMD_GROUPS, CMD_GROUPS2,
	CMD_INFO, CMD_INFO2,
	CMD_UPDATE, CMD_UPDATE2,
};
static char commands[][16]={
	"exit", "x",
	"say", "s",
	"to", "t",
	"help",
	"status",
	"enter", "e",
	"leave", "l",
	"who", "w",
	"view", "v",
	"groups", "g",
	"info", "i",
	"update", "u",
};

static char help_msg[]=
"Welcome to play MyQQ version 1.0beta\n"
"This is a tiny console IM, made for programmers and computer fans. Enjoy it!\n"
"help: Show this message.\n"
"to/t [UID|N]: Say something to UID:NUMBER.\n"
"say/s MESSAGE: Send message to who you are talking to.\n"
"status [online|away|hidden]: Change status.\n"
"who/w: List all online buddies. List online group members if you are a group.\n"
"view/v: List all buddies. List all group members if you are in a group.\n"
"enter/e [GID|N]: Enter a group, and you can talk to members by command 'say'\n"
"leave/l: Leave a group.\n"
"groups/g: Show all groups.\n"
"info/i: Show the information of a buddy or a group. Use 'to' or 'enter' fisrt.\n"
"update/u: Update the buddy list and the group list.\n" 
"exit/x: Logout.\n"
;


#define RESET_INPUT \
	if( need_reset ){\
	if( enter ){ \
		MSG("In {%s}> ", myqq_get_group_name( qid, group_int_uid ) ); \
	}else{ \
		MSG("To [%s]> ", myqq_get_buddy_name( qid, to_uid ) );} \
	fflush( stdout ); \
	need_reset = 0;}



static int getline(char *s, int lim) {
    char *t;
    int c;

    t = s;
    while (--lim>1 && (c=getchar()) != EOF && c != '\n')
        *s++ = c;
    *s = '\0';
    return s - t;
}

static char* skip_line( char* p, int ln )
{
	while( *p && ln-- )
	{
		p ++;
		while( *p && *p!='\n' ) p++;
	}
	return p;
}

static void STDCALL buddy_msg_callback ( uint uid, char* nick, time_t t, char* msg )
{
	char timestr[12];
	struct tm * timeinfo;
  	timeinfo = localtime ( &t );
	strftime( timestr, 12, "%H:%M:%S", timeinfo );
	char tmp[20], *p;
	sprintf( tmp, "%-16d", uid );
	p = strstr( buddy_buf, tmp );
	if( p )
	{
		p -= 8;
		if( p>=buddy_buf )
		{
			int ln;
			sscanf( p, "%d", &ln );
			MSG( "\n%d)%s[%s] %s\n", ln, timestr, nick, msg );
			RESET_INPUT
		}
	}else{
		MSG( "\n%s[%s] %s\n", timestr, nick, msg );
		RESET_INPUT
	}
}

static void STDCALL group_msg_callback ( uint uid, char* nick, uint int_uid, char* group_name,
	time_t t, char* msg )
{
	char timestr[12];
	struct tm * timeinfo;
  	timeinfo = localtime ( &t );
	strftime( timestr, 12, "%H:%M:%S", timeinfo );
	char tmp[20], *p;
	sprintf( tmp, "%-16d", int_uid );
	p = strstr( group_buf, tmp );
	if( p )
	{
		p -= 8;
		if( p>=group_buf )
		{
			int ln;
			sscanf( p, "%d", &ln );
			MSG("\n%d)%s{%s}-[%s] %s\n", ln, timestr, group_name, nick, msg );
			RESET_INPUT
		}
	}else{
		MSG("\n%s{%s}-[%s] %s\n", timestr, group_name, nick, msg );
		RESET_INPUT
	}
}


int main(int argc, char** argv)
{
	//init data
	qid = myqq_init();
	if( qid == 0 )
		return -1;
	//login
	if( argc<3 )
	{
		uint uid;
		char password[32];
		MSG("Please input your QQ ID: ");
		scanf("%d", &uid );
		MSG("Please input you Password: ");
		scanf("%s", password );
		MSG("Login in Hidden Mode? (y/n) ");
		scanf("%s", input );
		myqq_login( qid, uid, password, *input=='y'? 1 : 0 );
		scanf("%c", input ); //If I don't add this, it will display '>' twice.
	}else{
		myqq_login( qid, atoi(argv[1]), argv[2], argc>3? atoi(argv[3]):0 );
	}
	
	if( myqq_get_status( qid )==LM_OFFLINE )
		return -1;
	MSG( help_msg );
	group_buf = (char*)malloc( GROUP_BUF_SIZE );
	buddy_buf = (char*)malloc( BUDDY_BUF_SIZE );
	myqq_delegate_buddy_msg( qid, buddy_msg_callback );
	myqq_delegate_group_msg( qid, group_msg_callback );
	int cmdid, lastcmd=-1, len;
	char cmd[16], arg[1008];
	MSG("Downloading buddy list and group list, please wait ...\n");
	myqq_wait( qid, 10 );
	myqq_get_group_list( qid, group_buf, GROUP_BUF_SIZE );
	myqq_get_buddy_list( qid, buddy_buf, BUDDY_BUF_SIZE, 1 );
	while( myqq_get_status( qid )!=LM_OFFLINE ){
		RESET_INPUT
		len = getline( input, 1023 );
		if( len < 1 ) continue;
		char* sp = strchr( input, ' ' );
		if( sp ){
			*sp = '\0';
			strncpy( cmd, input, 16-1 );
			strncpy( arg, sp+1, 1008-1 );
			*sp = ' ';
		}else{
			strncpy( cmd, input, 16-1 );
			arg[0] = '\0';
		}
		need_reset = 1;
		for( cmdid=0; cmdid<sizeof(commands)/16; cmdid++ )
			if( strcmp( commands[cmdid], cmd )==0 )
				break;
SELECT_CMD:
		switch( cmdid ){
		case CMD_TO:
		case CMD_TO2:
		{
			if( enter )
			{
				MSG("You are in a group, you talk to everybody.\n");
				break;
			}
			int n = atoi( arg );
			if( n < 0xFFFF )
			{
				char *p;
				p = skip_line( buddy_buf, n-1 );
				if( p )
				{
					sscanf( p, "%d%d", &n, &to_uid );
					MSG("You are talking to %s\n", myqq_get_buddy_name(qid, to_uid) );
					break;
				}
			}else{
				to_uid = n;
				MSG("You are talking to %s\n", myqq_get_buddy_name(qid, to_uid) );
				break;
			}
			MSG("to: %s is not found.\n", arg );
			break;
		}
		case CMD_SAY:
		case CMD_SAY2:
		{
			if( enter )
			{
				if( myqq_send_im_to_group( qid, group_int_uid, arg, 1 ) < 0 ){
					MSG("TIMEOUT: You message has not been sent.\n");
				}
			}else{
				if( to_uid == 0 )
				{
					MSG("say: Say to who?\n");
					break;
				}
				if( myqq_send_im_to_buddy( qid, to_uid, arg, 1 ) < 0 ){
					MSG("TIMEOUT: You message has not been sent.\n");
				}
			}
			break;
		}
		case CMD_EXIT:
		case CMD_EXIT2:
			myqq_logout( qid );
			MSG("Logout.\n");
			exit(0);
			break;
		case CMD_HELP:
			MSG( help_msg );
			break;
		case CMD_STATUS:
			if( strcmp( arg, "away") == 0 )
				myqq_change_status( qid, LM_AWAY );
			else if( strcmp( arg, "online") == 0 )
				myqq_change_status( qid, LM_NORMAL );
			else if( strcmp( arg, "hidden") == 0 )
				myqq_change_status( qid, LM_HIDDEN );
			else{
				MSG("Unknown status %s\n", arg );
			}
			break;
		case CMD_ENTER:
		case CMD_ENTER2:
		{
			int n = atoi( arg );
			if( n < 0xFFFF )
			{
				char *p;
				p = skip_line( group_buf, n-1 );
				if( p )
				{
					sscanf( p, "%d%d", &n, &group_int_uid );
					MSG("You are in Group %s\n", myqq_get_group_name( qid, group_int_uid) );
					enter = 1;
					break;
				}
			}else{
				group_int_uid = n;
				MSG("You are in Group %s\n", myqq_get_group_name( qid, group_int_uid) );
				enter = 1;
				break;
			}
			MSG("enter: %s is not found.\n", arg );
			break;
		}
		case CMD_LEAVE:
		case CMD_LEAVE2:
			if( !enter )
			{
				MSG("Not entered.\n");
				break;
			}
			enter = 0;
			MSG("Leave %s.", myqq_get_group_name( qid, group_int_uid ) );
			MSG("You are talking to %s\n", myqq_get_buddy_name( qid, to_uid ) );
			break;
		case CMD_GROUPS:
		case CMD_GROUPS2:
		{
			myqq_get_group_list( qid, group_buf, GROUP_BUF_SIZE );
			MSG( group_buf );
			break;
		}
		case CMD_UPDATE:
		case CMD_UPDATE2:
			myqq_update_buddy_list( qid );
			myqq_update_group_list( qid );
			MSG("Updating...\n");
			if( myqq_wait( qid, 15 )<0 ){
				MSG("Failed to update.\n");
			}
			break;
		case CMD_INFO:
		case CMD_INFO2:
		{
			if( !enter )
			{
				if( to_uid==0 )
				{
					MSG("Please choose a buddy first.\n");
				}else{
					char* buf = (char*)malloc(1024*4);	//4kb enough!
					if( myqq_get_buddy_info( qid, to_uid, buf, 1024*4 ) < 0 ){
						MSG("Fail to get infor for %s\n",
							myqq_get_buddy_name( qid, to_uid ) );
					}else{
						MSG( buf );
					}
					free(buf);
				}
			}else{
				char* buf = (char*)malloc(1024*4);	//4kb enough!
				if( myqq_get_group_info( qid, group_int_uid, buf, 1024*4 ) < 0 ){
					MSG("Fail to get infor for %s\n",
						myqq_get_group_name( qid, group_int_uid ) );
				}else{
					MSG( buf );
				}
				free(buf);
			}
			break;
		}
		case CMD_VIEW:
		case CMD_VIEW2:
			if( enter )
			{
				myqq_get_group_member_list( qid, group_int_uid, buddy_buf,
					BUDDY_BUF_SIZE, 0 );
				MSG( buddy_buf );
			}else{
				myqq_get_buddy_list( qid, buddy_buf, BUDDY_BUF_SIZE, 0 );
				MSG( buddy_buf );
			}
			break;
		case CMD_WHO:
		case CMD_WHO2:
			if( enter )
			{
				myqq_get_group_member_list( qid, group_int_uid, buddy_buf,
					GROUP_BUF_SIZE, 1 );
				MSG( buddy_buf );
			}else{
				myqq_get_buddy_list( qid, buddy_buf, GROUP_BUF_SIZE, 1 );
				MSG( buddy_buf );
			}
			break;
		default:
			//use it as the last cmd's argument
			if( lastcmd && *input )
			{
				cmdid = lastcmd;
				strncpy( arg, input, 1008-1 );
				*input = 0;
				goto SELECT_CMD;
			}
			break;
		}
		lastcmd = cmdid;
	}
	myqq_cleanup( qid );
	return 0;
}
