#ifndef _PACKET_H
#define _PACKET_H

#include <sys/types.h>
#include "myqq.h"


#define PACKET_BUF_SIZE 20
#define MAX_PACKETS		256
#define MAX_PACKET_SIZE	65536

#define PACKET_IN	1
#define PACKET_OUT	2


typedef struct __attribute__ ((packed)) myqq_prot_packet_s{
	char		headflag;	//0x02 normally
	ushort		version;
	ushort		command;
	ushort		seqno;
	union{
		uint	uid;
		char	data_in[1];
	};
	char		data_out[1];
}myqq_prot_packet_t;

typedef struct myqq_packet_item_s{
	myqq_prot_packet_t*	packet;
	int			type;	//PACKET_IN or PACKET_OUT
	time_t			time;
	int			send_times;
	int			packet_len;
}myqq_packet_item_t;

//Packet structure
//ͬһʱֻһ߳޸ݡ
typedef struct myqq_packet_list_s{
	int			count;		//number of packet in list
	pthread_mutex_t	mutex;		//mutex for correctly accessing
	myqq_packet_item_t	items[MAX_PACKETS];	//packet list storing pointers
}myqq_packet_list_t;

myqq_prot_packet_t* packet_alloc( int qid, int len, int type );
void packet_free( int qid, myqq_prot_packet_t* packet );
int packet_send( int qid, myqq_prot_packet_t* packet, int len );
int packet_recv( int qid, myqq_prot_packet_t* packet, int len );
void packet_list_init( int qid );
void packet_list_cleanup( int qid );
myqq_prot_packet_t* packet_match( int qid, myqq_prot_packet_t* p1 );
myqq_prot_packet_t* packet_timeout( int qid, int secs, int* len, int* send_times );

#endif	//_PACKET_H
