#ifndef _PROTOCOL_H
#define _PROTOCOL_H

#include <sys/types.h>
#include "packet.h"

#define PROT_VERSION	0x1b0e 
#define PROT_TAIL	0x03
#define PROT_HEAD	0x02

// list of known QQ commands
// Copied from GNU openQ.
enum {
	QQ_CMD_LOGOUT = 0x0001,				/*+ log out */
	QQ_CMD_KEEP_ALIVE = 0x0002,				/*+ get onlines from tencent */
	QQ_CMD_UPDATE_INFO = 0x0004,			/* update information */
	QQ_CMD_SEARCH_USER = 0x0005,			/* search for user */
	QQ_CMD_GET_USER_INFO = 0x0006,			/*+ get user information */
	QQ_CMD_ADD_FRIEND_WO_AUTH = 0x0009,		/* add friend without auth */
	QQ_CMD_DEL_FRIEND = 0x000a,				/* delete a friend  */
	QQ_CMD_BUDDY_AUTH = 0x000b,				/* buddy authentication */
	QQ_CMD_CHANGE_ONLINE_STATUS = 0x000d,		/*+ change my online status */
	QQ_CMD_ACK_SYS_MSG = 0x0012,			/* ack system message */
	QQ_CMD_SEND_IM = 0x0016,				/*+ send message */
	QQ_CMD_RECV_IM = 0x0017,				/*+ receive message */
	QQ_CMD_REMOVE_SELF = 0x001c,			/* remove self */
	QQ_CMD_REQUEST_KEY = 0x001d,			/* request key for file transfer */
	QQ_CMD_CELL_PHONE_1 = 0x0021,			/* cell phone 1 */
	QQ_CMD_LOGIN = 0x0022,				/*+ login */
	QQ_CMD_GET_FRIENDS_LIST = 0x0026,			/*+ retrieve my freinds list */
	QQ_CMD_GET_FRIENDS_ONLINE = 0x0027,		/*+ get my online friends list */
	QQ_CMD_CELL_PHONE_2 = 0x0029,			/* cell phone 2 */
	QQ_CMD_GROUP_CMD = 0x0030,				/* group command */
	QQ_CMD_GET_ALL_LIST_WITH_GROUP = 0x0058,  	/* group.c for getting group list */
	QQ_CMD_GET_LEVEL = 0x005C,				/* get level for one or more buddies */
	QQ_CMD_REQUEST_LOGIN_TOKEN  = 0x0062, 		/*+ get login token */
	QQ_CMD_RECV_MSG_SYS = 0x0080,			/* receive a system message */
	QQ_CMD_RECV_MSG_FRIEND_CHANGE_STATUS = 0x0081,	/*+ friends change status */
};
enum {
	QQ_IM_TEXT = 0x01,
	QQ_IM_AUTO_REPLY = 0x02
};
enum {
	QQ_RECV_IM_TO_BUDDY = 0x0009,
	QQ_RECV_IM_TO_UNKNOWN = 0x000a,
	QQ_RECV_IM_NEWS = 0x0018,
	QQ_RECV_IM_UNKNOWN_QUN_IM = 0x0020,
	QQ_RECV_IM_ADD_TO_QUN = 0x0021,
	QQ_RECV_IM_DEL_FROM_QUN = 0x0022,
	QQ_RECV_IM_APPLY_ADD_TO_QUN = 0x0023,
	QQ_RECV_IM_APPROVE_APPLY_ADD_TO_QUN = 0x0024,
	QQ_RECV_IM_REJCT_APPLY_ADD_TO_QUN = 0x0025,
	QQ_RECV_IM_CREATE_QUN = 0x0026,
	QQ_RECV_IM_TEMP_QUN_IM = 0x002A,
	QQ_RECV_IM_QUN_IM = 0x002B,
	QQ_RECV_IM_SYS_NOTIFICATION = 0x0030
};
enum {
	QQ_NORMAL_IM_TEXT = 0x000b,
	QQ_NORMAL_IM_FILE_REQUEST_TCP = 0x0001,
	QQ_NORMAL_IM_FILE_APPROVE_TCP = 0x0003,
	QQ_NORMAL_IM_FILE_REJECT_TCP = 0x0005,
	QQ_NORMAL_IM_FILE_REQUEST_UDP = 0x0035,
	QQ_NORMAL_IM_FILE_APPROVE_UDP = 0x0037,
	QQ_NORMAL_IM_FILE_REJECT_UDP = 0x0039,
	QQ_NORMAL_IM_FILE_NOTIFY = 0x003b,
	QQ_NORMAL_IM_FILE_PASV = 0x003f,
	QQ_NORMAL_IM_FILE_CANCEL = 0x0049,
	QQ_NORMAL_IM_FILE_EX_REQUEST_UDP = 0x81,
	QQ_NORMAL_IM_FILE_EX_REQUEST_ACCEPT = 0x83,
	QQ_NORMAL_IM_FILE_EX_REQUEST_CANCEL = 0x85,
	QQ_NORMAL_IM_FILE_EX_NOTIFY_IP = 0x87
};

enum {
	QQ_BUDDY_OFFLINE = 0x00,
	QQ_BUDDY_ONLINE_NORMAL = 0x0a,
	QQ_BUDDY_ONLINE_OFFLINE = 0x14,
	QQ_BUDDY_ONLINE_AWAY = 0x1e,
	QQ_BUDDY_ONLINE_INVISIBLE = 0x28
};

enum {
	QQ_RECV_SYS_IM_KICK_OUT = 0x01
};


//
//ͷеimͬġ
typedef struct __attribute__((packed)) myqq_prot_recv_im_s {
	union{
		uint	sender_uid;
		uint	int_uid;
	};
	uint		recver_uid;
	uint		seqno;
	uint		sender_ip;
	ushort		sender_port;
	ushort		type;	//type of message //0x002b
}myqq_prot_recv_im_t;

void login_recv_info( int qid, char* data, int len );
void login_change_status_comp( int qid, char* data, int len );
void buddy_get_info( int qid, uint uid );
void buddy_recv_info( int qid, char* data, int len );
void buddy_get_list( int qid, int pos );
void buddy_recv_list( int qid, char* data, int len );
void buddy_get_online_list( int qid, int pos );
void buddy_recv_online_list( int qid, char* data, int len );
void buddy_recv_status( int qid, char * data, int len );
void buddy_recv_im( int qid, char* data, int len );
void buddy_send_text_im( int qid, uint uid, char* msg );
void myqq_refresh_buddy_list( int qid );
void im_recv( int qid, char* data, int len, myqq_prot_packet_t* packet );
void im_send( int qid, uint uid, char* content, int len );
void im_send_comp( int qid, char* data, int len );
void group_recv_im( int qid, char* data, int len, int cmd );
void group_get_list( int qid, uint pos );
void group_recv_list( int qid, char* data, int len );
void group_get_info( int qid, uint int_uid );
void group_recv_info( int qid, char* data, int len );
void group_get_online_list( int qid, uint int_uid );
void group_get_members_info( int qid, uint int_uid );
void cs_recv_keep_alive( int qid, char* data, int len );

#endif //_PROTOCOL_H

