/*
 *  list.c
 *
 *  Do with array list.
 *
 *  Copyright (C) 2008  Huang Guan
 *
 *  2008-02-5 Created.
 *  Note: Don't use these functions while another is using it..
 *        No any mutual symbols support.
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "myqq.h"
#include "debug.h"
#include "rlist.h"


//
int rlist_init( myqq_rlist_t* l, int size )
{
	if( l == NULL ) return -EFAULT;
	l->list = (int*) malloc( size * sizeof(int) );
	l->size = size;
	l->head = 0;
	l->tail = 0;
	return 0;
}

//
#define NEXT ( (l->tail+1)%l->size )
int rlist_append( myqq_rlist_t* l, int v )
{
	if( l == NULL ) return -EFAULT;
	//set value
	l->list[ l->tail ] = v;
	//move
	if( NEXT == l->head ) //full?
	{
		l->head = (l->head + 1)%l->size;
	}
	l->tail = NEXT;
	return 0;
}

//
void rlist_cleanup( myqq_rlist_t* l )
{
	if( l )
		free( l->list ); 
}

//
int rlist_find( myqq_rlist_t * l, int v )
{
	int i;
	if( l == NULL ) return -EFAULT;
	for( i=l->head; i!=l->tail; i=(i+1)%l->size )
	{
		if( l->list[i] == v )
			return 1;
	}
	return 0;
}

